@@ -1,5 +1,6 @@
 /* md.c  -  message digest dispatcher
- * Copyright (C) 1998, 1999, 2002, 2003 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2002, 2003, 2006,
+ *               2008 Free Software Foundation, Inc.
  *
  * This file is part of Libgcrypt.
  *
@@ -14,8 +15,7 @@
  * GNU Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * License along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -23,7 +23,6 @@
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
-#include <assert.h>
 
 #include "g10lib.h"
 #include "cipher.h"
@@ -31,38 +30,67 @@
 
 #include "rmd.h"
 
+/* A dummy extraspec so that we do not need to tests the extraspec
+   field from the module specification against NULL and instead
+   directly test the respective fields of extraspecs.  */
+static md_extra_spec_t dummy_extra_spec;
+
+
+/* This is the list of the digest implementations included in
+   libgcrypt.  */
 static struct digest_table_entry
 {
   gcry_md_spec_t *digest;
+  md_extra_spec_t *extraspec;
   unsigned int algorithm;
+  int fips_allowed; 
 } digest_table[] =
   {
 #if USE_CRC    
-    { &_gcry_digest_spec_crc32, GCRY_MD_CRC32 },
-    { &_gcry_digest_spec_crc32_rfc1510, GCRY_MD_CRC32_RFC1510 },
-    { &_gcry_digest_spec_crc24_rfc2440, GCRY_MD_CRC24_RFC2440 },
+    /* We allow the CRC algorithms even in FIPS mode because they are
+       actually no cryptographic primitives.  */
+    { &_gcry_digest_spec_crc32,   
+      &dummy_extra_spec,                 GCRY_MD_CRC32, 1 },
+    { &_gcry_digest_spec_crc32_rfc1510,  
+      &dummy_extra_spec,                 GCRY_MD_CRC32_RFC1510, 1 },
+    { &_gcry_digest_spec_crc24_rfc2440,
+      &dummy_extra_spec,                 GCRY_MD_CRC24_RFC2440, 1 },
 #endif
 #if USE_MD4
-    { &_gcry_digest_spec_md4, GCRY_MD_MD4 },
+    { &_gcry_digest_spec_md4,
+      &dummy_extra_spec,                 GCRY_MD_MD4 },
 #endif
 #if USE_MD5
-    { &_gcry_digest_spec_md5, GCRY_MD_MD5 },
+    { &_gcry_digest_spec_md5,
+      &dummy_extra_spec,                 GCRY_MD_MD5, 1 },
 #endif
 #if USE_RMD160
-    { &_gcry_digest_spec_rmd160, GCRY_MD_RMD160 },
+    { &_gcry_digest_spec_rmd160,
+      &dummy_extra_spec,                 GCRY_MD_RMD160 },
 #endif
 #if USE_SHA1
-    { &_gcry_digest_spec_sha1, GCRY_MD_SHA1 },
+    { &_gcry_digest_spec_sha1, 
+      &_gcry_digest_extraspec_sha1,      GCRY_MD_SHA1, 1 },
 #endif
 #if USE_SHA256
-    { &_gcry_digest_spec_sha256, GCRY_MD_SHA256 },
+    { &_gcry_digest_spec_sha256,
+      &_gcry_digest_extraspec_sha256,    GCRY_MD_SHA256, 1 },
+    { &_gcry_digest_spec_sha224,
+      &_gcry_digest_extraspec_sha224,    GCRY_MD_SHA224, 1 },
 #endif
 #if USE_SHA512
-    { &_gcry_digest_spec_sha512, GCRY_MD_SHA512 },
-    { &_gcry_digest_spec_sha384, GCRY_MD_SHA384 },
+    { &_gcry_digest_spec_sha512,
+      &_gcry_digest_extraspec_sha512,    GCRY_MD_SHA512, 1 },
+    { &_gcry_digest_spec_sha384,
+      &_gcry_digest_extraspec_sha384,    GCRY_MD_SHA384, 1 },
 #endif
 #if USE_TIGER
-    { &_gcry_digest_spec_tiger, GCRY_MD_TIGER },
+    { &_gcry_digest_spec_tiger,
+      &dummy_extra_spec,                 GCRY_MD_TIGER },
+#endif
+#if USE_WHIRLPOOL
+    { &_gcry_digest_spec_whirlpool,
+      &dummy_extra_spec,                 GCRY_MD_WHIRLPOOL },
 #endif
     { NULL },
   };
@@ -97,6 +125,7 @@ struct gcry_md_context
   int finalized;
   GcryDigestEntry *list;
   byte *macpads;
+  int macpads_Bsize;             /* Blocksize as used for the HMAC pads. */
 };
 
 
@@ -110,7 +139,7 @@ struct gcry_md_context
       ath_mutex_lock (&digests_registered_lock);   \
       if (! default_digests_registered)            \
         {                                          \
-          gcry_md_register_default ();             \
+          md_register_default ();                  \
           default_digests_registered = 1;          \
         }                                          \
       ath_mutex_unlock (&digests_registered_lock); \
@@ -125,14 +154,14 @@ static gcry_err_code_t md_open (gcry_md_hd_t *h, int algo,
 static gcry_err_code_t md_enable (gcry_md_hd_t hd, int algo);
 static gcry_err_code_t md_copy (gcry_md_hd_t a, gcry_md_hd_t *b);
 static void md_close (gcry_md_hd_t a);
-static void md_write (gcry_md_hd_t a, byte *inbuf, size_t inlen);
+static void md_write (gcry_md_hd_t a, const void *inbuf, size_t inlen);
 static void md_final(gcry_md_hd_t a);
 static byte *md_read( gcry_md_hd_t a, int algo );
 static int md_get_algo( gcry_md_hd_t a );
 static int md_digest_length( int algo );
 static const byte *md_asn_oid( int algo, size_t *asnlen, size_t *mdlen );
-static void md_start_debug( gcry_md_hd_t a, char *suffix );
-static void md_stop_debug( gcry_md_hd_t a );
+static void md_start_debug ( gcry_md_hd_t a, const char *suffix );
+static void md_stop_debug ( gcry_md_hd_t a );
 
 
 
@@ -140,16 +169,28 @@ static void md_stop_debug( gcry_md_hd_t a );
 /* Internal function.  Register all the ciphers included in
    CIPHER_TABLE.  Returns zero on success or an error code.  */
 static void
-gcry_md_register_default (void)
+md_register_default (void)
 {
   gcry_err_code_t err = 0;
   int i;
   
-  for (i = 0; (! err) && digest_table[i].digest; i++)
-    err = _gcry_module_add (&digests_registered,
-			    digest_table[i].algorithm,
-			    (void *) digest_table[i].digest,
-			    NULL);
+  for (i = 0; !err && digest_table[i].digest; i++)
+    {
+      if ( fips_mode ())
+        {
+          if (!digest_table[i].fips_allowed)
+            continue;
+          if (digest_table[i].algorithm == GCRY_MD_MD5
+              && _gcry_enforced_fips_mode () )
+            continue;  /* Do not register in enforced fips mode.  */
+        }
+
+      err = _gcry_module_add (&digests_registered,
+                              digest_table[i].algorithm,
+                              (void *) digest_table[i].digest,
+                              (void *) digest_table[i].extraspec,
+                              NULL);
+    }
 
   if (err)
     BUG ();
@@ -212,16 +253,23 @@ gcry_md_lookup_oid (const char *oid)
    DIGEST.  On success, a new algorithm ID is stored in ALGORITHM_ID
    and a pointer representhing this module is stored in MODULE.  */
 gcry_error_t
-gcry_md_register (gcry_md_spec_t *digest,
-		  unsigned int *algorithm_id,
-		  gcry_module_t *module)
+_gcry_md_register (gcry_md_spec_t *digest,
+                   md_extra_spec_t *extraspec,
+                   unsigned int *algorithm_id,
+                   gcry_module_t *module)
 {
   gcry_err_code_t err = 0;
   gcry_module_t mod;
 
+  /* We do not support module loading in fips mode.  */
+  if (fips_mode ())
+    return gpg_error (GPG_ERR_NOT_SUPPORTED);
+
   ath_mutex_lock (&digests_registered_lock);
   err = _gcry_module_add (&digests_registered, 0,
-			  (void *) digest, &mod);
+			  (void *) digest, 
+			  (void *)(extraspec? extraspec : &dummy_extra_spec), 
+                          &mod);
   ath_mutex_unlock (&digests_registered_lock);
   
   if (! err)
@@ -298,7 +346,7 @@ gcry_md_map_name (const char *string)
   ret = search_oid (string, &algorithm, NULL);
   if (! ret)
     {
-      /* Not found, search for an acording diget name.  */
+      /* Not found, search a matching digest name.  */
       digest = gcry_md_lookup_name (string);
       if (digest)
 	{
@@ -426,18 +474,28 @@ md_open (gcry_md_hd_t *h, int algo, int secure, int hmac)
 
       if (hmac)
 	{
-	  ctx->macpads = gcry_malloc_secure (128);
-	  if (! ctx->macpads)
+	  switch (algo)
+            {
+              case GCRY_MD_SHA384:
+              case GCRY_MD_SHA512:
+                ctx->macpads_Bsize = 128;
+                break;
+              default:
+                ctx->macpads_Bsize = 64;
+                break;
+            }
+          ctx->macpads = gcry_malloc_secure (2*(ctx->macpads_Bsize));
+	  if (!ctx->macpads)
 	    {
-	      md_close (hd);
 	      err = gpg_err_code_from_errno (errno);
+	      md_close (hd);
 	    }
 	}
     }
 
   if (! err)
     {
-      /* FIXME: should we really do that? - yes [-wk] */
+      /* Hmmm, should we really do that? - yes [-wk] */
       _gcry_fast_random_poll ();
 
       if (algo)
@@ -502,10 +560,22 @@ md_enable (gcry_md_hd_t hd, int algorithm)
       log_debug ("md_enable: algorithm %d not available\n", algorithm);
       err = GPG_ERR_DIGEST_ALGO;
     }
-  else
+ else
     digest = (gcry_md_spec_t *) module->spec;
 
-  if (! err)
+  
+  if (!err && algorithm == GCRY_MD_MD5 && fips_mode ())
+    {
+      _gcry_inactivate_fips_mode ("MD5 used");
+      if (_gcry_enforced_fips_mode () )
+        {
+          /* We should never get to here because we do not register
+             MD5 in enforced fips mode. But better throw an error.  */
+          err = GPG_ERR_DIGEST_ALGO;
+        }
+    }
+  
+  if (!err)
     {
       size_t size = (sizeof (*entry)
                      + digest->contextsize
@@ -549,8 +619,7 @@ md_enable (gcry_md_hd_t hd, int algorithm)
 gcry_error_t
 gcry_md_enable (gcry_md_hd_t hd, int algorithm)
 {
-  gcry_err_code_t err = md_enable (hd, algorithm);
-  return gcry_error (err);
+  return gcry_error (md_enable (hd, algorithm));
 }
 
 static gcry_err_code_t
@@ -579,54 +648,63 @@ md_copy (gcry_md_hd_t ahd, gcry_md_hd_t *b_hd)
     {
       bhd->ctx = b = (struct gcry_md_context *) ((char *) bhd + n);
       /* No need to copy the buffer due to the write above. */
-      assert (ahd->bufsize == (n - sizeof (struct gcry_md_handle) + 1));
+      gcry_assert (ahd->bufsize == (n - sizeof (struct gcry_md_handle) + 1));
       bhd->bufsize = ahd->bufsize;
       bhd->bufpos = 0;
-      assert (! ahd->bufpos);
+      gcry_assert (! ahd->bufpos);
       memcpy (b, a, sizeof *a);
       b->list = NULL;
       b->debug = NULL;
       if (a->macpads)
 	{
-	  b->macpads = gcry_malloc_secure (128);
+	  b->macpads = gcry_malloc_secure (2*(a->macpads_Bsize));
 	  if (! b->macpads)
 	    {
-	      md_close (bhd);
 	      err = gpg_err_code_from_errno (errno);
+	      md_close (bhd);
 	    }
 	  else
-	    memcpy (b->macpads, a->macpads, 128);
+	    memcpy (b->macpads, a->macpads, (2*(a->macpads_Bsize)));
 	}
     }
 
   /* Copy the complete list of algorithms.  The copied list is
      reversed, but that doesn't matter. */
-  if (! err)
-    for (ar = a->list; ar; ar = ar->next)
-      {
-	if (a->secure)
-	  br = gcry_xmalloc_secure (sizeof *br
-				    + ar->digest->contextsize
-				    - sizeof(ar->context));
-	else
-	  br = gcry_xmalloc (sizeof *br
-			     + ar->digest->contextsize
-			     - sizeof (ar->context));
-	memcpy (br, ar,
-		sizeof (*br) + ar->digest->contextsize - sizeof (ar->context));
-	br->next = b->list;
-	b->list = br;
+  if (!err)
+    {
+      for (ar = a->list; ar; ar = ar->next)
+        {
+          if (a->secure)
+            br = gcry_malloc_secure (sizeof *br
+                                     + ar->digest->contextsize
+                                     - sizeof(ar->context));
+          else
+            br = gcry_malloc (sizeof *br
+                              + ar->digest->contextsize
+                              - sizeof (ar->context));
+          if (!br)
+            {
+	      err = gpg_err_code_from_errno (errno);
+              md_close (bhd);
+              break;
+            }
 
-	/* Add a reference to the module.  */
-	ath_mutex_lock (&digests_registered_lock);
-	_gcry_module_use (br->module);
-	ath_mutex_unlock (&digests_registered_lock);
-       }
+          memcpy (br, ar, (sizeof (*br) + ar->digest->contextsize
+                           - sizeof (ar->context)));
+          br->next = b->list;
+          b->list = br;
+          
+          /* Add a reference to the module.  */
+          ath_mutex_lock (&digests_registered_lock);
+          _gcry_module_use (br->module);
+          ath_mutex_unlock (&digests_registered_lock);
+        }
+    }
 
-  if (a->debug)
+  if (a->debug && !err)
     md_start_debug (bhd, "unknown");
 
-  if (! err)
+  if (!err)
     *b_hd = bhd;
 
   return err;
@@ -635,7 +713,9 @@ md_copy (gcry_md_hd_t ahd, gcry_md_hd_t *b_hd)
 gcry_error_t
 gcry_md_copy (gcry_md_hd_t *handle, gcry_md_hd_t hd)
 {
-  gcry_err_code_t err = md_copy (hd, handle);
+  gcry_err_code_t err;
+
+  err = md_copy (hd, handle);
   if (err)
     *handle = NULL;
   return gcry_error (err);
@@ -649,7 +729,9 @@ void
 gcry_md_reset (gcry_md_hd_t a)
 {
   GcryDigestEntry *r;
-  
+
+  /* Note: We allow this even in fips non operational mode.  */
+
   a->bufpos = a->ctx->finalized = 0;
 
   for (r = a->ctx->list; r; r = r->next)
@@ -658,7 +740,7 @@ gcry_md_reset (gcry_md_hd_t a)
       (*r->digest->init) (&r->context.c);
     }
   if (a->ctx->macpads)
-    md_write (a, a->ctx->macpads, 64); /* inner pad */
+    md_write (a, a->ctx->macpads, a->ctx->macpads_Bsize); /* inner pad */
 }
 
 static void
@@ -682,7 +764,7 @@ md_close (gcry_md_hd_t a)
 
   if (a->ctx->macpads)
     {
-      wipememory (a->ctx->macpads, 128);
+      wipememory (a->ctx->macpads, 2*(a->ctx->macpads_Bsize));
       gcry_free(a->ctx->macpads);
     }
 
@@ -693,11 +775,12 @@ md_close (gcry_md_hd_t a)
 void
 gcry_md_close (gcry_md_hd_t hd)
 {
+  /* Note: We allow this even in fips non operational mode.  */
   md_close (hd);
 }
 
 static void
-md_write (gcry_md_hd_t a, byte *inbuf, size_t inlen)
+md_write (gcry_md_hd_t a, const void *inbuf, size_t inlen)
 {
   GcryDigestEntry *r;
   
@@ -721,7 +804,7 @@ md_write (gcry_md_hd_t a, byte *inbuf, size_t inlen)
 void
 gcry_md_write (gcry_md_hd_t hd, const void *inbuf, size_t inlen)
 {
-  md_write (hd, (unsigned char *) inbuf, inlen);
+  md_write (hd, inbuf, inlen);
 }
 
 static void
@@ -751,7 +834,9 @@ md_final (gcry_md_hd_t a)
 
       if (err)
 	_gcry_fatal_error (err, NULL);
-      md_write (om, a->ctx->macpads+64, 64);
+      md_write (om, 
+                (a->ctx->macpads)+(a->ctx->macpads_Bsize), 
+                a->ctx->macpads_Bsize);
       md_write (om, p, dlen);
       md_final (om);
       /* Replace our digest with the mac (they have the same size). */
@@ -761,44 +846,44 @@ md_final (gcry_md_hd_t a)
 }
 
 static gcry_err_code_t
-prepare_macpads( gcry_md_hd_t hd, const byte *key, size_t keylen)
+prepare_macpads (gcry_md_hd_t hd, const unsigned char *key, size_t keylen)
 {
   int i;
-  int algo = md_get_algo( hd );
-  byte *helpkey = NULL;
-  byte *ipad, *opad;
+  int algo = md_get_algo (hd);
+  unsigned char *helpkey = NULL;
+  unsigned char *ipad, *opad;
 
-  if ( !algo )
-    return GPG_ERR_DIGEST_ALGO; /* i.e. no algo enabled */
+  if (!algo)
+    return GPG_ERR_DIGEST_ALGO; /* Might happen if no algo is enabled.  */
 
-  if ( keylen > 64 ) 
+  if ( keylen > hd->ctx->macpads_Bsize ) 
     {
-      helpkey = gcry_malloc_secure ( md_digest_length( algo ) );
-      if ( !helpkey )
+      helpkey = gcry_malloc_secure (md_digest_length (algo));
+      if (!helpkey)
         return gpg_err_code_from_errno (errno);
-      gcry_md_hash_buffer ( algo, helpkey, key, keylen );
+      gcry_md_hash_buffer (algo, helpkey, key, keylen);
       key = helpkey;
-      keylen = md_digest_length( algo );
-      assert ( keylen <= 64 );
+      keylen = md_digest_length (algo);
+      gcry_assert ( keylen <= hd->ctx->macpads_Bsize );
     }
 
-  memset ( hd->ctx->macpads, 0, 128 );
+  memset ( hd->ctx->macpads, 0, 2*(hd->ctx->macpads_Bsize) );
   ipad = hd->ctx->macpads;
-  opad = hd->ctx->macpads+64;
+  opad = (hd->ctx->macpads)+(hd->ctx->macpads_Bsize);
   memcpy ( ipad, key, keylen );
   memcpy ( opad, key, keylen );
-  for (i=0; i < 64; i++ ) 
+  for (i=0; i < hd->ctx->macpads_Bsize; i++ ) 
     {
       ipad[i] ^= 0x36;
       opad[i] ^= 0x5c;
     }
-  gcry_free( helpkey );
+  gcry_free (helpkey);
 
   return GPG_ERR_NO_ERROR;
 }
 
 gcry_error_t
-gcry_md_ctl (gcry_md_hd_t hd, int cmd, byte *buffer, size_t buflen)
+gcry_md_ctl (gcry_md_hd_t hd, int cmd, void *buffer, size_t buflen)
 {
   gcry_err_code_t rc = 0;
   
@@ -814,7 +899,7 @@ gcry_md_ctl (gcry_md_hd_t hd, int cmd, byte *buffer, size_t buflen)
       md_start_debug (hd, buffer);
       break;
     case GCRYCTL_STOP_DUMP:
-      md_stop_debug( hd );
+      md_stop_debug ( hd );
       break;
     default:
       rc = GPG_ERR_INV_OP;
@@ -827,7 +912,7 @@ gcry_md_setkey (gcry_md_hd_t hd, const void *key, size_t keylen)
 {
   gcry_err_code_t rc = GPG_ERR_NO_ERROR;
 
-  if (! hd->ctx->macpads)
+  if (!hd->ctx->macpads)
     rc = GPG_ERR_CONFLICT;
   else
     {
@@ -839,6 +924,19 @@ gcry_md_setkey (gcry_md_hd_t hd, const void *key, size_t keylen)
   return gcry_error (rc);
 }
 
+/* The new debug interface.  If SUFFIX is a string it creates an debug
+   file for the context HD.  IF suffix is NULL, the file is closed and
+   debugging is stopped.  */
+void
+gcry_md_debug (gcry_md_hd_t hd, const char *suffix)
+{
+  if (suffix)
+    md_start_debug (hd, suffix);
+  else
+    md_stop_debug (hd);
+}
+
+
 
 /****************
  * if ALGO is null get the digest for the used algo (which should be only one)
@@ -852,7 +950,7 @@ md_read( gcry_md_hd_t a, int algo )
     {
       /* return the first algorithm */
       if (r && r->next)
-	log_debug("more than algorithm in md_read(0)\n");
+	log_debug ("more than one algorithm in md_read(0)\n");
       return r->digest->read( &r->context.c );
     }
   else
@@ -872,69 +970,27 @@ md_read( gcry_md_hd_t a, int algo )
 byte *
 gcry_md_read (gcry_md_hd_t hd, int algo)
 {
+  /* This function is expected to always return a digest, thus we
+     can't return an error which we actually should do in
+     non-operational state.  */
   gcry_md_ctl (hd, GCRYCTL_FINALIZE, NULL, 0);
   return md_read (hd, algo);
 }
 
-/****************
- * This function combines md_final and md_read but keeps the context
- * intact.  This function can be used to calculate intermediate
- * digests.  The digest is copied into buffer and the digestlength is
- * returned.  If buffer is NULL only the needed size for buffer is returned.
- * buflen gives the max size of buffer. If the buffer is too shourt to
- * hold the complete digest, the buffer is filled with as many bytes are
- * possible and this value is returned.
- */
-#if 0
-static int
-md_digest( gcry_md_hd_t a, int algo, byte *buffer, int buflen )
-{
-  struct md_digest_list_s *r = NULL;
-  char *context;
-  char *digest;
-
-  if( a->bufpos )
-    md_write( a, NULL, 0 );
-
-  if( !algo ) {  /* return digest for the first algorithm */
-    if( (r=a->ctx->list) && r->next )
-      log_debug("more than algorithm in md_digest(0)\n");
-  }
-  else {
-    for(r=a->ctx->list; r; r = r->next )
-      if( r->algo == algo )
-	break;
-  }
-  if( !r )
-    BUG();
-
-  if( !buffer )
-    return r->mdlen;
-
-  /* I don't want to change the interface, so I simply work on a copy
-   * of the context (extra overhead - should be fixed)*/
-  context = a->ctx->secure ? gcry_xmalloc_secure( r->contextsize )
-    : gcry_xmalloc( r->contextsize );
-  memcpy( context, r->context.c, r->contextsize );
-  (*r->digest->final)( context );
-  digest = (*r->digest->read)( context );
-
-  if( buflen > r->mdlen )
-    buflen = r->mdlen;
-  memcpy( buffer, digest, buflen );
-
-  gcry_free(context);
-  return buflen;
-}
-#endif
 
 /*
- * Read out an intermediate digest.  Not yet fucntional.
+ * Read out an intermediate digest.  Not yet functional.
  */
 gcry_err_code_t
 gcry_md_get (gcry_md_hd_t hd, int algo, byte *buffer, int buflen)
 {
+  (void)hd;
+  (void)algo;
+  (void)buffer;
+  (void)buflen;
+
   /*md_digest ... */
+  fips_signal_error ("unimplemented function called");
   return GPG_ERR_INTERNAL;
 }
 
@@ -951,14 +1007,27 @@ gcry_md_hash_buffer (int algo, void *digest,
 {
   if (algo == GCRY_MD_SHA1)
     _gcry_sha1_hash_buffer (digest, buffer, length);
-  else if (algo == GCRY_MD_RMD160)
+  else if (algo == GCRY_MD_RMD160 && !fips_mode () )
     _gcry_rmd160_hash_buffer (digest, buffer, length);
   else
     {
       /* For the others we do not have a fast function, so we use the
 	 normal functions. */
       gcry_md_hd_t h;
-      gpg_err_code_t err = md_open (&h, algo, 0, 0);
+      gpg_err_code_t err;
+
+      if (algo == GCRY_MD_MD5 && fips_mode ())
+        {
+          _gcry_inactivate_fips_mode ("MD5 used");
+          if (_gcry_enforced_fips_mode () )
+            {
+              /* We should never get to here because we do not register
+                 MD5 in enforced fips mode.  */
+              _gcry_fips_noreturn ();
+            }
+        }
+
+      err = md_open (&h, algo, 0, 0);
       if (err)
 	log_bug ("gcry_md_open failed for algo %d: %s",
                  algo, gpg_strerror (gcry_error(err)));
@@ -975,7 +1044,10 @@ md_get_algo (gcry_md_hd_t a)
   GcryDigestEntry *r = a->ctx->list;
 
   if (r && r->next)
-    log_error("WARNING: more than algorithm in md_get_algo()\n");
+    {
+      fips_signal_error ("possible usage error");
+      log_error ("WARNING: more than one algorithm in md_get_algo()\n");
+    }
   return r ? r->module->mod_id : 0;
 }
 
@@ -1081,27 +1153,31 @@ gcry_md_algo_info (int algo, int what, void *buffer, size_t *nbytes)
       break;
 
     case GCRYCTL_GET_ASNOID:
-      {
-	const char unsigned *asn;
-	size_t asnlen;
-
-	asn = md_asn_oid (algo, &asnlen, NULL);
-	if (buffer && (*nbytes >= asnlen))
+      /* We need to check that the algo is available because
+         md_asn_oid would otherwise raise an assertion. */
+      err = check_digest_algo (algo);
+      if (!err)
+        {
+          const char unsigned *asn;
+          size_t asnlen;
+          
+          asn = md_asn_oid (algo, &asnlen, NULL);
+          if (buffer && (*nbytes >= asnlen))
 	  {
 	    memcpy (buffer, asn, asnlen);
 	    *nbytes = asnlen;
 	  }
-	else if ((! buffer) && nbytes)
-	  *nbytes = asnlen;
-	else
-	  {
-	    if (buffer)
-	      err = GPG_ERR_TOO_SHORT;
-	    else
-	      err = GPG_ERR_INV_ARG;
-	  }
-	break;
-      }
+          else if (!buffer && nbytes)
+            *nbytes = asnlen;
+          else
+            {
+              if (buffer)
+                err = GPG_ERR_TOO_SHORT;
+              else
+                err = GPG_ERR_INV_ARG;
+            }
+        }
+      break;
 
   default:
     err = GPG_ERR_INV_OP;
@@ -1112,31 +1188,37 @@ gcry_md_algo_info (int algo, int what, void *buffer, size_t *nbytes)
 
 
 static void
-md_start_debug( gcry_md_hd_t md, char *suffix )
+md_start_debug ( gcry_md_hd_t md, const char *suffix )
 {
   static int idx=0;
   char buf[50];
 
-  if( md->ctx->debug ) {
-    log_debug("Oops: md debug already started\n");
+  if (fips_mode ())
     return;
-  }
+  
+  if ( md->ctx->debug )
+    {
+      log_debug("Oops: md debug already started\n");
+      return;
+    }
   idx++;
-  sprintf(buf, "dbgmd-%05d.%.10s", idx, suffix );
+  snprintf (buf, DIM(buf)-1, "dbgmd-%05d.%.10s", idx, suffix );
   md->ctx->debug = fopen(buf, "w");
-  if( !md->ctx->debug )
+  if ( !md->ctx->debug )
     log_debug("md debug: can't open %s\n", buf );
 }
 
 static void
 md_stop_debug( gcry_md_hd_t md )
 {
-  if( md->ctx->debug ) {
-    if( md->bufpos )
-      md_write( md, NULL, 0 );
-    fclose(md->ctx->debug);
-    md->ctx->debug = NULL;
-  }
+  if ( md->ctx->debug )
+    {
+      if ( md->bufpos )
+        md_write ( md, NULL, 0 );
+      fclose (md->ctx->debug);
+      md->ctx->debug = NULL;
+    }
+
 #ifdef HAVE_U64_TYPEDEF
   {  /* a kludge to pull in the __muldi3 for Solaris */
     volatile u32 a = (u32)(ulong)md;
@@ -1155,7 +1237,7 @@ md_stop_debug( gcry_md_hd_t md )
  *	Returns 1 when the handle works on secured memory
  *	otherwise 0 is returned.  There is no error return.
  *  GCRYCTL_IS_ALGO_ENABLED:
- *     Returns 1 if the algo is enanled for that handle.
+ *     Returns 1 if the algo is enabled for that handle.
  *     The algo must be passed as the address of an int.
  */
 gcry_error_t
@@ -1199,6 +1281,8 @@ gcry_md_info (gcry_md_hd_t h, int cmd, void *buffer, size_t *nbytes)
   return gcry_error (err);
 }
 
+
+/* Explicitly initialize this module.  */
 gcry_err_code_t
 _gcry_md_init (void)
 {
@@ -1251,3 +1335,41 @@ gcry_md_list (int *list, int *list_length)
 
   return err;
 }
+
+
+/* Run the selftests for digest algorithm ALGO with optional reporting
+   function REPORT.  */
+gpg_error_t
+_gcry_md_selftest (int algo, int extended, selftest_report_func_t report)
+{
+  gcry_module_t module = NULL;
+  cipher_extra_spec_t *extraspec = NULL;
+  gcry_err_code_t ec = 0;
+
+  REGISTER_DEFAULT_DIGESTS;
+
+  ath_mutex_lock (&digests_registered_lock);
+  module = _gcry_module_lookup_id (digests_registered, algo);
+  if (module && !(module->flags & FLAG_MODULE_DISABLED))
+    extraspec = module->extraspec;
+  ath_mutex_unlock (&digests_registered_lock);
+  if (extraspec && extraspec->selftest)
+    ec = extraspec->selftest (algo, extended, report);
+  else
+    {
+      ec = GPG_ERR_DIGEST_ALGO;
+      if (report)
+        report ("digest", algo, "module", 
+                module && !(module->flags & FLAG_MODULE_DISABLED)?
+                "no selftest available" :
+                module? "algorithm disabled" : "algorithm not found");
+    }
+
+  if (module)
+    {
+      ath_mutex_lock (&digests_registered_lock);
+      _gcry_module_release (module);
+      ath_mutex_unlock (&digests_registered_lock);
+    }
+  return gpg_error (ec);
+}
